##############################################################################
#
# Copyright (c) 2007 Nexedi SA and Contributors. All Rights Reserved.
#
# This program is Free Software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
##############################################################################

from ctypes.util import find_library
from ctypes import cdll
glpk_lib = cdll.LoadLibrary(find_library('glpk'))

LPX_LP          = 100
LPX_MIP         = 101
LPX_FR          = 110
LPX_LO          = 111
LPX_UP          = 112
LPX_DB          = 113
LPX_FX          = 114
LPX_MIN         = 120
LPX_MAX         = 121
LPX_B_UNDEF     = 130
LPX_B_VALID     = 131
LPX_P_UNDEF     = 132
LPX_P_FEAS      = 133
LPX_P_INFEAS    = 134
LPX_P_NOFEAS    = 135
LPX_D_UNDEF     = 136
LPX_D_FEAS      = 137
LPX_D_INFEAS    = 138
LPX_D_NOFEAS    = 139
LPX_BS          = 140
LPX_NL          = 141
LPX_NU          = 142
LPX_NF          = 143
LPX_NS          = 144
LPX_T_UNDEF     = 150
LPX_T_OPT       = 151
LPX_CV          = 160
LPX_IV          = 161
LPX_I_UNDEF     = 170
LPX_I_OPT       = 171
LPX_I_FEAS      = 172
LPX_I_NOFEAS    = 173
LPX_OPT         = 180
LPX_FEAS        = 181
LPX_INFEAS      = 182
LPX_NOFEAS      = 183
LPX_UNBND       = 184
LPX_UNDEF       = 185
LPX_E_OK        = 200
LPX_E_EMPTY     = 201
LPX_E_BADB      = 202
LPX_E_INFEAS    = 203
LPX_E_FAULT     = 204
LPX_E_OBJLL     = 205
LPX_E_OBJUL     = 206
LPX_E_ITLIM     = 207
LPX_E_TMLIM     = 208
LPX_E_NOFEAS    = 209
LPX_E_INSTAB    = 210
LPX_E_SING      = 211
LPX_E_NOCONV    = 212
LPX_E_NOPFS     = 213
LPX_E_NODFS     = 214
LPX_K_MSGLEV    = 300
LPX_K_SCALE     = 301
LPX_K_DUAL      = 302
LPX_K_PRICE     = 303
LPX_K_RELAX     = 304
LPX_K_TOLBND    = 305
LPX_K_TOLDJ     = 306
LPX_K_TOLPIV    = 307
LPX_K_ROUND     = 308
LPX_K_OBJLL     = 309
LPX_K_OBJUL     = 310
LPX_K_ITLIM     = 311
LPX_K_ITCNT     = 312
LPX_K_TMLIM     = 313
LPX_K_OUTFRQ    = 314
LPX_K_OUTDLY    = 315
LPX_K_BRANCH    = 316
LPX_K_BTRACK    = 317
LPX_K_TOLINT    = 318
LPX_K_TOLOBJ    = 319
LPX_K_MPSINFO   = 320
LPX_K_MPSOBJ    = 321
LPX_K_MPSORIG   = 322
LPX_K_MPSWIDE   = 323
LPX_K_MPSFREE   = 324
LPX_K_MPSSKIP   = 325
LPX_K_LPTORIG   = 326
LPX_K_PRESOL    = 327
LPX_K_BINARIZE  = 328
LPX_K_USECUTS   = 329
LPX_C_COVER     = 0x01
LPX_C_CLIQUE    = 0x02
LPX_C_GOMORY    = 0x04
LPX_C_ALL       = 0xFF

method_list = (
    'glp_lpx_create_prob',
    'glp_lpx_set_prob_name',
    'glp_lpx_set_obj_name',
    'glp_lpx_set_obj_dir',
    'glp_lpx_add_rows',
    'glp_lpx_add_cols',
    'glp_lpx_set_row_name',
    'glp_lpx_set_col_name',
    'glp_lpx_set_row_bnds',
    'glp_lpx_set_col_bnds',
    'glp_lpx_set_obj_coef',
    'glp_lpx_set_mat_row',
    'glp_lpx_set_mat_col',
    'glp_lpx_load_matrix',
    'glp_lpx_del_rows',
    'glp_lpx_del_cols',
    'glp_lpx_delete_prob',
    'glp_lpx_get_prob_name',
    'glp_lpx_get_obj_name',
    'glp_lpx_get_obj_dir',
    'glp_lpx_get_num_rows',
    'glp_lpx_get_num_cols',
    'glp_lpx_get_row_name',
    'glp_lpx_get_col_name',
    'glp_lpx_get_row_type',
    'glp_lpx_get_row_lb',
    'glp_lpx_get_row_ub',
    'glp_lpx_get_row_bnds',
    'glp_lpx_get_col_type',
    'glp_lpx_get_col_lb',
    'glp_lpx_get_col_ub',
    'glp_lpx_get_col_bnds',
    'glp_lpx_get_obj_coef',
    'glp_lpx_get_num_nz',
    'glp_lpx_get_mat_row',
    'glp_lpx_get_mat_col',
    'glp_lpx_create_index',
    'glp_lpx_find_row',
    'glp_lpx_find_col',
    'glp_lpx_delete_index',
    'glp_lpx_scale_prob',
    'glp_lpx_unscale_prob',
    'glp_lpx_std_basis',
    'glp_lpx_adv_basis',
    'glp_lpx_cpx_basis',
    'glp_lpx_set_row_stat',
    'glp_lpx_set_col_stat',
    'glp_lpx_simplex',
    'glp_lpx_exact',
    'glp_lpx_get_status',
    'glp_lpx_get_prim_stat',
    'glp_lpx_get_dual_stat',
    'glp_lpx_get_obj_val',
    'glp_lpx_get_row_stat',
    'glp_lpx_get_row_prim',
    'glp_lpx_get_row_dual',
    'glp_lpx_get_row_info',
    'glp_lpx_get_col_stat',
    'glp_lpx_get_col_prim',
    'glp_lpx_get_col_dual',
    'glp_lpx_get_col_info',
    'glp_lpx_get_ray_info',
    'glp_lpx_check_kkt',
    'glp_lpx_warm_up',
    'glp_lpx_eval_tab_row',
    'glp_lpx_eval_tab_col',
    'glp_lpx_transform_row',
    'glp_lpx_transform_col',
    'glp_lpx_prim_ratio_test',
    'glp_lpx_dual_ratio_test',
    'glp_lpx_interior',
    'glp_lpx_ipt_status',
    'glp_lpx_ipt_obj_val',
    'glp_lpx_ipt_row_prim',
    'glp_lpx_ipt_row_dual',
    'glp_lpx_ipt_col_prim',
    'glp_lpx_ipt_col_dual',
    'glp_lpx_set_class',
    'glp_lpx_get_class',
    'glp_lpx_set_col_kind',
    'glp_lpx_get_col_kind',
    'glp_lpx_get_num_int',
    'glp_lpx_get_num_bin',
    'glp_lpx_integer',
    'glp_lpx_intopt',
    'glp_lpx_mip_status',
    'glp_lpx_mip_obj_val',
    'glp_lpx_mip_row_val',
    'glp_lpx_mip_col_val',
    'glp_lpx_check_int',
    'glp_lpx_reset_parms',
    'glp_lpx_set_int_parm',
    'glp_lpx_get_int_parm',
    'glp_lpx_set_real_parm',
    'glp_lpx_get_real_parm',
    'glp_lpx_read_mps',
    'glp_lpx_write_mps',
    'glp_lpx_read_bas',
    'glp_lpx_write_bas',
    'glp_lpx_read_freemps',
    'glp_lpx_write_freemps',
    'glp_lpx_read_cpxlp',
    'glp_lpx_write_cpxlp',
    'glp_lpx_read_model',
    'glp_lpx_print_prob',
    'glp_lpx_print_sol',
    'glp_lpx_print_sens_bnds',
    'glp_lpx_print_ips',
    'glp_lpx_print_mip',
    'glp_lpx_is_b_avail',
    'glp_lpx_write_pb',
    'glp_lpx_main',
)

import sys
for method in method_list:
    func = getattr(glpk_lib, '_%s' % method, None )
    if func is not None:
        setattr(sys.modules.get(__name__), method, func)
    else:
        func = getattr(glpk_lib, method, None )
        if func is not None:
            setattr(sys.modules.get(__name__), method, func)
del find_library, cdll, glpk_lib, sys, method, method_list, func
